@@ -1,11 +1,11 @@
-#![doc(html_root_url = "https://docs.rs/sarif-fmt/0.2.11")]
+#![doc(html_root_url = "https://docs.rs/sarif-fmt/0.2.17")]
 #![recursion_limit = "256"]
 //! # WARNING: VERY UNSTABLE (EARLY IMPLEMENTATION)
 //!
 //! This crate provides a command line tool to pretty print SARIF files to
 //! easy human readable output.
 //!
-//! The latest [documentation can be found here](https://psastras.github.io/sarif-rs/sarif_fmt/index.html).
+//! The latest [documentation can be found here](https://docs.rs/sarif_fmt).
 //!
 //! SARIF or the Static Analysis Results Interchange Format is an industry
 //! standard format for the output of static analysis tools. More information
@@ -37,7 +37,7 @@
 //!     │ ╰───────────────────────────────────────────────────────^
 //!     │  
 //!     = `#[warn(clippy::bind_instead_of_map)]` on by default
-//!       for further information visit https://rust-lang.github.io/rust-clippy/master/index.html#bind_instead_of_map
+//!       for further information visit https://rust-lang.github.io/rust-clippy/master#bind_instead_of_map
 //! ```
 
 use anyhow::Result;
@@ -465,8 +465,87 @@ fn resolve_short_description_from_result(
     })
 }
 
+fn to_writer_plain(sarif: &sarif::Sarif) -> Result<()> {
+  let mut files = SimpleFiles::new();
+  sarif.runs.iter().try_for_each(|run| -> Result<()> {
+    let mut diagnostics = vec![];
+    if let Some(results) = run.results.as_ref() {
+      results.iter().try_for_each(|result| -> Result<()> {
+        let rules = vec![];
+        let rules = run.tool.driver.rules.as_ref().unwrap_or(&rules);
+        let level = resolve_level(rules, run, result);
+
+        if let (Some(text), Some(locations)) = (
+          resolve_message_text_from_result(result, run),
+          result.locations.as_ref(),
+        ) {
+          locations.iter().for_each(|location| {
+            if let Some((file_id, range)) = location
+              .physical_location
+              .as_ref()
+              .and_then(|physical_location| {
+                physical_location.artifact_location.as_ref().and_then(
+                  |artifact_location| {
+                    artifact_location.uri.as_ref().and_then(|uri| {
+                      physical_location.region.as_ref().and_then(|region| {
+                        get_physical_location_contents(physical_location, run)
+                          .ok()
+                          .and_then(|contents| {
+                            let file_id = files.add(uri, contents);
+                            if let (Some(range_start), Some(range_end)) =
+                              get_byte_range(file_id, &files, region)
+                            {
+                              Some((file_id, range_start..range_end))
+                            } else {
+                              None
+                            }
+                          })
+                      })
+                    })
+                  },
+                )
+              })
+            {
+              if let (Ok(name), Ok(location)) =
+                (files.name(file_id), files.location(file_id, range.start))
+              {
+                let diagnostic = (
+                  name.clone(),
+                  level,
+                  location.line_number,
+                  location.column_number,
+                  text.clone(),
+                );
+                diagnostics.push(diagnostic);
+              } else {
+                // todo: no location found
+              }
+            }
+          });
+        }
+
+        Ok(())
+      })?;
+    };
+
+    diagnostics.sort_by(|a, b| {
+      a.0.partial_cmp(&b.0).unwrap_or(std::cmp::Ordering::Equal)
+    });
+
+    diagnostics.iter().for_each(|diagnostic| {
+      println!(
+        "{} [{}:{}:{}] {}",
+        diagnostic.1, diagnostic.0, diagnostic.2, diagnostic.3, diagnostic.4
+      )
+    });
+    Ok(())
+  })?;
+
+  Ok(())
+}
+
 fn to_writer_pretty(sarif: &sarif::Sarif) -> Result<()> {
-  let mut writer = StandardStream::stderr(ColorChoice::Always);
+  let mut writer = StandardStream::stdout(ColorChoice::Auto);
   let mut files = SimpleFiles::new();
   let config = codespan_reporting::term::Config::default();
   let mut message_counter = (0, 0, 0);
@@ -588,6 +667,7 @@ fn to_writer_pretty(sarif: &sarif::Sarif) -> Result<()> {
     writer.write_all(
       format!(": {} warnings emitted\n", message_counter.1).as_bytes(),
     )?;
+    writer.reset()?;
   }
 
   if message_counter.2 > 0 {
@@ -599,6 +679,7 @@ fn to_writer_pretty(sarif: &sarif::Sarif) -> Result<()> {
     writer.write_all(
       format!(": {} errors emitted\n", message_counter.1).as_bytes(),
     )?;
+    writer.reset()?;
   }
 
   Ok(())
@@ -611,6 +692,15 @@ fn main() -> Result<()> {
             "The expected input is generated by running 'cargo clippy --message-format=json'.",
         )
         .version(env!("CARGO_PKG_VERSION"))
+
+        .arg(
+          Arg::new("message format").about("One of plain or pretty")
+            .default_value("pretty")
+            .short('f')
+            .long("message-format")
+            .value_name("FMT")
+            .takes_value(true)
+        )
         .arg(
             Arg::new("input")
                 .about("input file; reads from stdin if none is given")
@@ -624,5 +714,8 @@ fn main() -> Result<()> {
   };
   let reader = BufReader::new(read);
   let sarif = process(reader)?;
-  to_writer_pretty(&sarif)
+  match matches.value_of("message format").unwrap_or("pretty") {
+    "plain" => to_writer_plain(&sarif),
+    _ => to_writer_pretty(&sarif),
+  }
 }
